blog_macros::posts!("blog/src/post_list");

use core::time::Duration;

use chrono::Datelike;

const VALID_CHARS_FOR_ROUTE_ID: &str =
    "01234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_";

#[derive(Debug, Clone)]
pub struct Post {
    pub title: &'static str,
    pub timestamp: u64,
    pub content: fn() -> yew::Html,
}

impl PartialEq for Post {
    fn eq(&self, other: &Self) -> bool {
        self.title == other.title && self.timestamp == other.timestamp
    }
}

impl Post {
    pub fn route_id(&self) -> String {
        let post_time = chrono::DateTime::UNIX_EPOCH + Duration::from_secs(self.timestamp);

        let mut route_id = self
            .title
            .replace(' ', "_")
            .matches(
                VALID_CHARS_FOR_ROUTE_ID
                    .chars()
                    .collect::<Vec<char>>()
                    .as_slice(),
            )
            .collect::<String>();
        route_id.truncate(30);

        format!("{}{:0>2}_{route_id}", post_time.year(), post_time.month())
    }
}
